@@ -1,3 +1,4 @@
+#!/usr/local/bin/python2.7
 # -*- coding: utf-8 -*-
 #
 # (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
@@ -25,24 +26,31 @@ import re
 import gzip
 import os.path
 import time
-import urllib # TODO: Replace with urllib2 (urllib is deprecated in Python 3.0)
-import StringIO
-import cStringIO
-import httplib
+from .sixext.moves import urllib_request, urllib_parse, urllib_error # TODO: Replace with urllib2 (urllib is deprecated in Python 3.0)
+import io
+from io import BytesIO
+from .sixext.moves import http_client
 import struct
 import string
-
+import time
 # Local
-from g import *
-from codes import *
-import utils
-import status
-import pml
-import status
+from .g import *
+from .codes import *
+from . import utils
+from . import services
+from . import os_utils
+from . import status
+from . import pml
+from . import status
 from prnt import pcl, ldl, cups
-import models, mdns, slp
-from strings import StringTable
+from . import models, mdns, slp, avahi
+from .strings import *
+from .sixext import PY3, to_bytes_utf8, to_unicode, to_string_latin, to_string_utf8, xStringIO
 
+http_result_pat = re.compile("""HTTP/\d.\d\s(\d+)""", re.I)
+
+HTTP_OK = 200
+HTTP_ERROR = 500
 
 try:
     import hpmudext
@@ -79,7 +87,7 @@ DEFAULT_FILTER = None
 VALID_FILTERS = ('print', 'scan', 'fax', 'pcard', 'copy')
 DEFAULT_BE_FILTER = ('hp',)
 
-pat_deviceuri = re.compile(r"""(.*):/(.*?)/(\S*?)\?(?:serial=(\S*)|device=(\S*)|ip=(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}[^&]*)|zc=(\S+))(?:&port=(\d))?""", re.IGNORECASE)
+pat_deviceuri = re.compile(r"""(.*):/(.*?)/(\S*?)\?(?:serial=(\S*)|device=(\S*)|ip=(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}[^&]*)|zc=(\S+)|hostname=(\S+))(?:&port=(\d))?""", re.IGNORECASE)
 http_pat_url = re.compile(r"""/(.*?)/(\S*?)\?(?:serial=(\S*)|device=(\S*))&loc=(\S*)""", re.IGNORECASE)
 direct_pat = re.compile(r'direct (.*?) "(.*?)" "(.*?)" "(.*?)"', re.IGNORECASE)
 
@@ -94,26 +102,18 @@ ip_pat = re.compile(r"""\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25
 dev_pat = re.compile(r"""/dev/.+""", re.IGNORECASE)
 usb_pat = re.compile(r"""(\d+):(\d+)""", re.IGNORECASE)
 
-### **********Lambda Function UniStar for checking type of arguments to constructor of class event*******************************
-
-UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')
-
-
-#
-# Event Wrapper Class for pipe IPC
-#
 
 class Event(object):
     def __init__(self, device_uri, printer_name, event_code,
                  username=prop.username, job_id=0, title='',
                  timedate=0):
-       # UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')
-        self.device_uri = UniStr(device_uri)
-        self.printer_name = UniStr(printer_name)
+
+        self.device_uri = to_unicode(device_uri)
+        self.printer_name = to_unicode(printer_name)
         self.event_code = int(event_code)
-        self.username = UniStr(username)
+        self.username = to_unicode(username)
         self.job_id = int(job_id)
-        self.title = UniStr(title)
+        self.title = to_unicode(title)
 
         if timedate:
             self.timedate = float(timedate)
@@ -135,8 +135,8 @@ class Event(object):
 
 
     def pack_for_pipe(self):
-        return struct.pack(self.pipe_fmt, self.device_uri, self.printer_name,
-                self.event_code, self.username, self.job_id, self.title,
+        return struct.pack(self.pipe_fmt, self.device_uri.encode('utf-8'), self.printer_name.encode('utf-8'),
+                self.event_code, self.username.encode('utf-8'), self.job_id, self.title.encode('utf-8'),
                 self.timedate)
 
 
@@ -245,11 +245,11 @@ def init_dbus(dbus_loop=None):
                 session_bus = dbus.SessionBus()
             else:
                 session_bus = dbus.SessionBus(dbus_loop)
-        except dbus.exceptions.DBusException, e:
+        except dbus.exceptions.DBusException as e:
             if os.getuid() != 0:
-                log.error("Unable to connect to dbus session bus.")
+                log.error("Unable to connect to dbus session bus. %s "%e)
             else:
-                log.debug("Unable to connect to dbus session bus (running as root?)")
+                log.debug("Unable to connect to dbus session bus (running as root?). %s "%e)
 
             dbus_avail = False
             return dbus_avail, None,  None
@@ -258,7 +258,7 @@ def init_dbus(dbus_loop=None):
             log.debug("Connecting to com.hplip.StatusService (try #1)...")
             service = session_bus.get_object('com.hplip.StatusService', "/com/hplip/StatusService")
             dbus_avail = True
-        except dbus.exceptions.DBusException, e:
+        except dbus.exceptions.DBusException as e:
             try:
                 os.waitpid(-1, os.WNOHANG)
             except OSError:
@@ -286,7 +286,7 @@ def init_dbus(dbus_loop=None):
                     log.debug("Connecting to com.hplip.StatusService (try #%d)..." % t)
                     service = session_bus.get_object('com.hplip.StatusService', "/com/hplip/StatusService")
 
-                except dbus.exceptions.DBusException, e:
+                except dbus.exceptions.DBusException as e:
                     log.debug("Unable to connect to dbus. Is hp-systray running?")
                     t += 1
 
@@ -318,6 +318,7 @@ def makeURI(param, port=1):
         result_code, uri = hpmudext.make_par_uri(param)
 
         if result_code == hpmudext.HPMUD_R_OK and uri:
+            uri = to_string_utf8(uri)
             log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
@@ -333,6 +334,7 @@ def makeURI(param, port=1):
         result_code, uri = hpmudext.make_usb_uri(usb_bus_id, usb_dev_id)
 
         if result_code == ERROR_SUCCESS and uri:
+            uri = to_string_utf8(uri)
             log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
@@ -345,6 +347,7 @@ def makeURI(param, port=1):
         result_code, uri = hpmudext.make_net_uri(param, port)
 
         if result_code == hpmudext.HPMUD_R_OK and uri:
+            uri = to_string_utf8(uri)
             log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
@@ -357,11 +360,24 @@ def makeURI(param, port=1):
         result_code, uri = hpmudext.make_zc_uri(param, port)
 
         if result_code == hpmudext.HPMUD_R_OK and uri:
+            uri = to_string_utf8(uri)
             log.debug("Found: %s" % uri)
             found = True
             cups_uri = uri
-        else:
-            log.debug("Not found.")
+
+        else: # Try DNS hostname
+            log.debug("Device not found using mDNS hostname. Trying with DNS hostname %s" % param)
+
+            result_code, uri = hpmudext.make_net_uri(param, port)
+
+            if result_code == hpmudext.HPMUD_R_OK and uri:
+                uri = to_string_utf8(uri)
+                uri = uri.replace("ip=","hostname=")
+                log.debug("Found: %s" % uri)
+                found = True
+                cups_uri = uri
+            else:
+                log.debug("Not found.")
 
     if not found:
         log.debug("Trying serial number %s" % param)
@@ -381,7 +397,7 @@ def makeURI(param, port=1):
                 mq = queryModelByURI(d)
 
                 result_code, device_id = \
-                    hpmudext.device_open(d, mq.get('io-mode', hpmudext.HPMUD_UNI_MODE))
+                    hpmudext.open_device(d, mq.get('io-mode', hpmudext.HPMUD_UNI_MODE))
 
                 if result_code == hpmudext.HPMUD_R_OK:
                     result_code, data = hpmudext.get_device_id(device_id)
@@ -399,7 +415,7 @@ def makeURI(param, port=1):
     if found:
         try:
             mq = queryModelByURI(cups_uri)
-        except Error, e:
+        except Error as e:
             log.error("Error: %s" % e.msg)
             cups_uri, sane_uri, fax_uri = '', '', ''
         else:
@@ -447,7 +463,7 @@ def queryModelByURI(device_uri):
 #
 
 def probeDevices(bus=DEFAULT_PROBE_BUS, timeout=10,
-                 ttl=4, filter=DEFAULT_FILTER,  search='', net_search='mdns',
+                 ttl=4, filter=DEFAULT_FILTER,  search='', net_search='slp',
                  back_end_filter=('hp',)):
 
     num_devices, ret_devices = 0, {}
@@ -469,14 +485,23 @@ def probeDevices(bus=DEFAULT_PROBE_BUS, timeout=10,
             if net_search == 'slp':
                 try:
                     detected_devices = slp.detectNetworkDevices(ttl, timeout)
-                except Error, socket.error:
-                    log.error("An error occured during network probe.")
+                except Error as socket_error:
+                    socket.error = socket_error
+                    log.error("An error occured during network probe.[%s]"%socket_error)
                     raise ERROR_INTERNAL
-            else:
+            elif net_search == 'avahi':
+                try:
+                    detected_devices = avahi.detectNetworkDevices(ttl, timeout)
+                except Error as socket_error:
+                    socket.error = socket_error
+                    log.error("An error occured during network probe.[%s]"%socket_error)
+                    raise ERROR_INTERNAL
+            else :#if net_search = 'mdns'
                 try:
                     detected_devices = mdns.detectNetworkDevices(ttl, timeout)
-                except Error, socket.error:
-                    log.error("An error occured during network probe.")
+                except Error as socket_error:
+                    socket.error = socket_error
+                    log.error("An error occured during network probe.[%s]"%socket_error)
                     raise ERROR_INTERNAL
 
             for ip in detected_devices:
@@ -528,7 +553,6 @@ def probeDevices(bus=DEFAULT_PROBE_BUS, timeout=10,
                 bn = hpmudext.HPMUD_BUS_USB
 
             result_code, data = hpmudext.probe_devices(bn)
-
             if result_code == hpmudext.HPMUD_R_OK:
                 for x in data.splitlines():
                     m = direct_pat.match(x)
@@ -627,6 +651,7 @@ def probeDevices(bus=DEFAULT_PROBE_BUS, timeout=10,
 def getSupportedCUPSDevices(back_end_filter=['hp'], filter=DEFAULT_FILTER):
     devices = {}
     printers = cups.getPrinters()
+    log.debug(printers)
 
     for p in printers:
         try:
@@ -698,9 +723,8 @@ def getSupportedCUPSPrinters(back_end_filter=['hp'], filter=DEFAULT_FILTER):
                 include = __checkFilter(filter, mq)
 
             if include:
-                p.name = p.name.decode('utf-8')
                 printer_list.append(p)
-            #printer_list[p.name] = p.device_uri
+
 
     return printer_list # [ cupsext.Printer, ... ]
 
@@ -795,7 +819,6 @@ def parseDynamicCounter(ctr_field, convert_to_int=True):
 
 def parseDeviceURI(device_uri):
     m = pat_deviceuri.match(device_uri)
-
     if m is None:
         log.debug("Device URI %s is invalid/unknown" % device_uri)
         raise Error(ERROR_INVALID_DEVICE_URI)
@@ -812,9 +835,14 @@ def parseDeviceURI(device_uri):
     serial = m.group(4) or ''
     dev_file = m.group(5) or ''
     host = m.group(6) or ''
-    zc = ''
-    if not host:
-        zc = host = m.group(7) or ''
+    zc = m.group(7) or ''
+    hostname = m.group(8) or ''
+
+    if hostname:
+        host = hostname
+    elif zc:
+        host = zc
+
     port = m.group(8) or 1
 
     if bus == 'net':
@@ -845,7 +873,7 @@ def isNetwork(bus):
 #
 
 def __checkFilter(filter, mq):
-    for f, p in filter.items():
+    for f, p in list(filter.items()):
         if f is not None:
             op, val = p
             if not op(mq[f], val):
@@ -880,77 +908,6 @@ def validateFilterList(filter):
     return True
 
 
-#
-# UI String Queries (why is this here?)
-#
-
-inter_pat = re.compile(r"""%(.*)%""", re.IGNORECASE)
-st = StringTable()
-strings_init = False
-
-
-def initStrings():
-    global strings_init, st
-    strings_init = True
-    cycles = 0
-
-    while True:
-        found = False
-
-        for s in st.string_table:
-            short_string, long_string = st.string_table[s]
-            short_replace, long_replace = short_string, long_string
-
-            try:
-                short_match = inter_pat.match(short_string).group(1)
-            except (AttributeError, TypeError):
-                short_match = None
-
-            if short_match is not None:
-                found = True
-
-                try:
-                    short_replace, dummy = st.string_table[short_match]
-                except KeyError:
-                    log.error("String interpolation error: %s" % short_match)
-
-            try:
-                long_match = inter_pat.match(long_string).group(1)
-            except (AttributeError, TypeError):
-                long_match = None
-
-            if long_match is not None:
-                found = True
-
-                try:
-                    dummy, long_replace = st.string_table[long_match]
-                except KeyError:
-                    log.error("String interpolation error: %s" % long_match)
-
-            if found:
-                st.string_table[s] = (short_replace, long_replace)
-
-        if not found:
-            break
-        else:
-            cycles +=1
-            if cycles > 1000:
-                break
-
-
-def queryString(string_id, typ=0):
-    if not strings_init:
-        initStrings()
-
-    #log.debug("queryString(%s)" % string_id)
-    s = st.string_table.get(str(string_id), ('', ''))[typ]
-
-    if type(s) == type(''):
-        return s
-
-    return s()
-
-
 AGENT_types = { AGENT_TYPE_NONE        : 'invalid',
                 AGENT_TYPE_BLACK       : 'black',
                 AGENT_TYPE_BLACK_B8800 : 'black',
@@ -974,6 +931,8 @@ AGENT_types = { AGENT_TYPE_NONE        : 'invalid',
                 AGENT_TYPE_PG          : 'photo_gray',
                 AGENT_TYPE_C_M         : 'cyan_and_magenta',
                 AGENT_TYPE_K_Y         : 'black_and_yellow',
+                AGENT_TYPE_PHOTO_BLACK : 'photo_black',
+                AGENT_TYPE_MATTE_BLACK : 'matte_black',
                 AGENT_TYPE_UNSPECIFIED : 'unspecified', # Kind=5,6
             }
 
@@ -1007,8 +966,6 @@ AGENT_levels = {AGENT_LEVEL_TRIGGER_MAY_BE_LOW : 'low',
                }
 
 
-#
-
 
 
 # **************************************************************************** #
@@ -1080,6 +1037,11 @@ class Device(object):
 
         log.debug("Model/UI model: %s/%s" % (self.model, self.model_ui))
 
+        if self.bus == 'net':
+            self.http_host = self.host
+        else:
+            self.http_host = 'localhost'  
+
         # TODO:
         #service.setAlertsEx(self.hpssd_sock)
 
@@ -1160,7 +1122,7 @@ class Device(object):
             try:
                 log.debug("Sending event %d to hpssd..." % event_code)
                 self.service.SendEvent(self.device_uri, printer_name, event_code, prop.username, job_id, title)
-            except dbus.exceptions.DBusException, e:
+            except dbus.exceptions.DBusException as e:
                 log.debug("dbus call to SendEvent() failed.")
 
 
@@ -1243,7 +1205,7 @@ class Device(object):
 
             if len(self.channels) > 0:
 
-                for c in self.channels.keys():
+                for c in list(self.channels.keys()):
                     self.__closeChannel(c)
 
             result_code = hpmudext.close_device(self.device_id)
@@ -1388,6 +1350,9 @@ class Device(object):
 
     def getDeviceID(self):
         needs_close = False
+        self.raw_deviceID = ''
+        self.deviceID = {}
+
         if self.io_state != IO_STATE_HP_OPEN:
            try:
                self.open()
@@ -1397,10 +1362,7 @@ class Device(object):
 
         result_code, data = hpmudext.get_device_id(self.device_id)
 
-        if result_code != hpmudext.HPMUD_R_OK:
-            self.raw_deviceID = ''
-            self.deviceID = {}
-        else:
+        if result_code == hpmudext.HPMUD_R_OK:
             self.raw_deviceID = data
             self.deviceID = parseDeviceID(data)
 
@@ -1465,7 +1427,7 @@ class Device(object):
                 if self.dbus_avail:
                     try:
                         r_value = int(self.service.GetCachedIntValue(self.device_uri, 'r_value'))
-                    except dbus.exceptions.DBusException, e:
+                    except dbus.exceptions.DBusException as e:
                         log.debug("dbus call to GetCachedIntValue() failed.")
                         r_value = -1
 
@@ -1492,7 +1454,7 @@ class Device(object):
                                 if self.dbus_avail:
                                     try:
                                         self.service.SetCachedIntValue(self.device_uri, 'r_value', r_value)
-                                    except dbus.exceptions.DBusException, e:
+                                    except dbus.exceptions.DBusException as e:
                                         log.debug("dbus call to SetCachedIntValue() failed.")
                             else:
                                 log.error("Error attempting to read r-value (2).")
@@ -1519,7 +1481,7 @@ class Device(object):
                             if self.dbus_avail:
                                 try:
                                     self.service.SetCachedIntValue(self.device_uri, 'r_value', r_value)
-                                except dbus.exceptions.DBusException, e:
+                                except dbus.exceptions.DBusException as e:
                                     log.debug("dbus call to SetCachedIntValue() failed.")
 
                         else:
@@ -1544,7 +1506,7 @@ class Device(object):
                 if self.tech_type in (TECH_TYPE_MONO_INK, TECH_TYPE_COLOR_INK):
                     try:
                         self.getDeviceID()
-                    except Error, e:
+                    except Error as e:
                         log.error("Error getting device ID.")
                         self.last_event = Event(self.device_uri, '', ERROR_DEVICE_IO_ERROR,
                             prop.username, 0, '', time.time())
@@ -1674,7 +1636,7 @@ class Device(object):
             if self.tech_type in (TECH_TYPE_MONO_INK, TECH_TYPE_COLOR_INK):
                 try:
                     self.getDeviceID()
-                except Error, e:
+                except Error as e:
                     log.error("Error getting device ID.")
                     self.last_event = Event(self.device_uri, '', ERROR_DEVICE_IO_ERROR,
                         prop.username, 0, '', time.time())
@@ -1749,21 +1711,6 @@ class Device(object):
 
             status_code = self.dq.get('status-code', STATUS_UNKNOWN)
 
-##            if not quick and \
-##                self.mq.get('fax-type', FAX_TYPE_NONE) and \
-##                status_code == STATUS_PRINTER_IDLE and \
-##                io_mode != IO_MODE_UNI:
-##
-##                log.debug("Fax activity check...")
-##
-##                tx_active, rx_active = status.getFaxStatus(self)
-##
-##                if tx_active:
-##                    status_code = STATUS_FAX_TX_ACTIVE
-##                elif rx_active:
-##                    status_code = STATUS_FAX_RX_ACTIVE
-
-
             self.error_state = STATUS_TO_ERROR_STATE_MAP.get(status_code, ERROR_STATE_CLEAR)
             self.error_code = status_code
             self.sendEvent(self.error_code)
@@ -1835,13 +1782,13 @@ class Device(object):
                             dynamic_sku_data = True
                             break
                     except:
-                        pass                                 
+                        pass
 
                 a, aa = 1, 1
                 while True:
                     if dynamic_sku_data:
                         if a > len(agents):
-                            break 
+                            break
                         agent = agents[a-1]
                         mq_agent_sku = agent['agent-sku']
                         agent_kind = agent['kind']
@@ -2031,10 +1978,8 @@ class Device(object):
 
     def getPML(self, oid, desired_int_size=pml.INT_SIZE_INT): # oid => ( 'dotted oid value', pml type )
         channel_id = self.openPML()
-
         result_code, data, typ, pml_result_code = \
             hpmudext.get_pml(self.device_id, channel_id, pml.PMLToSNMP(oid[0]), oid[1])
-
         if pml_result_code > pml.ERROR_MAX_OK:
             log.debug("PML/SNMP GET %s failed (result code = 0x%x)" % (oid[0], pml_result_code))
             return pml_result_code, None
@@ -2050,15 +1995,12 @@ class Device(object):
             else:
                 log.debug("PML/SNMP GET %s (result code = 0x%x) returned: %s" %
                     (oid[0], pml_result_code, repr(converted_data)))
-
         return pml_result_code, converted_data
 
 
     def setPML(self, oid, value): # oid => ( 'dotted oid value', pml type )
         channel_id = self.openPML()
-
         value = pml.ConvertToPMLDataFormat(value, oid[1])
-
         result_code, pml_result_code = \
             hpmudext.set_pml(self.device_id, channel_id, pml.PMLToSNMP(oid[0]), oid[1], value)
 
@@ -2067,10 +2009,9 @@ class Device(object):
 
                 log.debug("PML/SNMP SET %s (result code = 0x%x) to:" %
                     (oid[0], pml_result_code))
-                log.log_data(value)
             else:
                 log.debug("PML/SNMP SET %s (result code = 0x%x) to: %s" %
-                    (oid[0], pml_result_code, repr(value)))
+                    (oid[0], pml_result_code, repr(value.decode('utf-8'))))
 
         return pml_result_code
 
@@ -2164,6 +2105,41 @@ class Device(object):
     def readWifiConfig(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openWifiConfig, bytes_to_read, stream, timeout, allow_short_read)
 
+#Common handling of reading chunked or unchunked data from LEDM devices
+    def readLEDMData(dev, func, reply, timeout=6):
+
+        END_OF_DATA=to_bytes_utf8("0\r\n\r\n")
+        bytes_requested = 1024
+        bytes_remaining = 0
+        chunkedFlag = True
+
+        bytes_read = func(bytes_requested, reply, timeout)
+
+        for line in reply.getvalue().splitlines():
+            if line.lower().find(to_bytes_utf8("content-length")) != -1:
+                 bytes_remaining = int(line.split(to_bytes_utf8(":"))[1])
+                 chunkedFlag = False
+                 break
+
+        xml_data_start = reply.getvalue().find(to_bytes_utf8("<?xml"))
+        if (xml_data_start != -1):
+            bytes_remaining = bytes_remaining - (len(reply.getvalue())  - xml_data_start)
+
+        while bytes_read > 0:
+            temp_buf = xStringIO()
+            bytes_read = func(bytes_requested, temp_buf, timeout)
+
+            reply.write(temp_buf.getvalue())
+
+            if not chunkedFlag:     # Unchunked data
+                bytes_remaining = bytes_remaining - bytes_read
+                if bytes_remaining <= 0:
+                    break
+            elif END_OF_DATA == temp_buf.getvalue():   # Chunked data end
+                    break
+
+
+
     def __readChannel(self, opener, bytes_to_read, stream=None,
                       timeout=prop.read_timeout, allow_short_read=False):
 
@@ -2175,7 +2151,7 @@ class Device(object):
         num_bytes = 0
 
         if stream is None:
-            buffer = ''
+            buffer = to_bytes_utf8('')
 
         while True:
             result_code, data = \
@@ -2198,7 +2174,7 @@ class Device(object):
                 break
 
             if stream is None:
-                buffer = ''.join([buffer, data])
+                buffer = to_bytes_utf8('').join([buffer, data])
             else:
                 stream.write(data)
 
@@ -2251,6 +2227,8 @@ class Device(object):
         return self.__writeChannel(self.openSoapFax, data)
 
     def writeMarvellFax(self, data):
+        if not isinstance(data, bytes) and hasattr(data, 'tobytes'):   # hasattr function used for supporting 2.6
+            data = data.tobytes()
         return self.__writeChannel(self.openMarvellFax, data)
 
     def writeWifiConfig(self, data):
@@ -2259,14 +2237,13 @@ class Device(object):
     def __writeChannel(self, opener, data):
         channel_id = opener()
         buffer, bytes_out, total_bytes_to_write = data, 0, len(data)
-
         log.debug("Writing %d bytes to channel %d (device-id=%d)..." % (total_bytes_to_write, channel_id, self.device_id))
 
         while len(buffer) > 0:
             result_code, bytes_written = \
-                hpmudext.write_channel(self.device_id, channel_id,
+                hpmudext.write_channel(self.device_id, channel_id, 
                     buffer[:prop.max_message_len])
-
+ 
             log.debug("Result code=%d" % result_code)
 
             if result_code != hpmudext.HPMUD_R_OK:
@@ -2296,10 +2273,54 @@ class Device(object):
                                                     value,
                                                     oid[1])))
 
-        log.log_data(data)
+        #log.log_data(data)
 
         self.printData(data, direct=direct, raw=True)
 
+    def post(self, url, post):
+        status_type = self.mq.get('status-type', STATUS_TYPE_NONE)
+        data = """POST %s HTTP/1.1\r
+Connection: Keep-alive\r
+User-agent: hplip/2.0\r
+Host: %s\r
+Content-type: text/xml\r
+Content-length: %d\r
+\r
+%s""" % (url, self.http_host, len(post), post)
+        log.log_data(data)
+        if status_type == STATUS_TYPE_LEDM:
+            log.debug("status-type: %d" % status_type)
+            self.writeEWS_LEDM(data)
+            response = BytesIO()
+
+            self.readLEDMData(self.readEWS_LEDM, response)
+
+            response = response.getvalue()
+            log.log_data(response)
+            self.closeEWS_LEDM()
+
+        elif status_type == STATUS_TYPE_LEDM_FF_CC_0:
+            log.debug("status-type: %d" % status_type)
+            self.writeLEDM(data)
+            response = BytesIO()
+
+            self.readLEDMData(self.readLEDM, response)
+
+            response = response.getvalue()
+            log.log_data(response)
+            self.closeLEDM()
+
+        else:
+            log.error("Not an LEDM status-type: %d" % status_type)
+
+        match = http_result_pat.match(to_string_utf8(response))
+        if match is None: return HTTP_OK
+        try:
+            code = int(match.group(1))
+        except (ValueError, TypeError):
+            code = HTTP_ERROR
+
+        return code == HTTP_OK
 
     def printGzipFile(self, file_name, printer_name=None, direct=False, raw=True, remove=False):
         return self.printFile(file_name, printer_name, direct, raw, remove)
@@ -2316,13 +2337,13 @@ class Device(object):
         f = gzip.open(print_file, 'r')
 
         x = f.readline()
-        while not x.startswith('%PY_BEGIN'):
+        while not x.startswith(to_bytes_utf8('%PY_BEGIN')):
             os.write(temp_file_fd, x)
             x = f.readline()
 
         sub_lines = []
         x = f.readline()
-        while not x.startswith('%PY_END'):
+        while not x.startswith(to_bytes_utf8('%PY_END')):
             sub_lines.append(x)
             x = f.readline()
 
@@ -2336,14 +2357,19 @@ class Device(object):
                  'DEVNODE' : self.dev_file,
                  }
 
-        if self.bus == 'net':
+        if self.bus == 'net' :
             SUBS['DEVNODE'] = 'n/a'
         else:
-            SUBS['IP'] = 'n/a'
+            SUBS['IP']= 'n/a'
             SUBS['PORT'] = 'n/a'
-
+        
+        if PY3:
+            sub_lines = [s.decode('utf-8') for s in sub_lines]
+        
+            
         for s in sub_lines:
-            os.write(temp_file_fd, s % SUBS)
+            os.write(temp_file_fd, to_bytes_utf8((s % SUBS)))
+        
 
         os.write(temp_file_fd, f.read())
         f.close()
@@ -2367,7 +2393,7 @@ class Device(object):
             if is_gzip:
                 self.writePrint(gzip.open(file_name, 'r').read())
             else:
-                self.writePrint(file(file_name, 'r').read())
+                self.writePrint(open(file_name, 'r').read())
 
         else:
             if not utils.which('lpr'):
@@ -2381,8 +2407,7 @@ class Device(object):
                 else:
                     c = 'lp -c -d%s %s %s' % (printer_name, lp_opt, file_name)
 
-                log.debug(c)
-                exit_code = os.system(c)
+                exit_code = os_utils.execute(c)
 
                 if exit_code != 0:
                     log.error("Print command failed with exit code %d!" % exit_code)
@@ -2400,8 +2425,7 @@ class Device(object):
                 else:
                     c = 'lpr -P%s %s %s %s' % (printer_name, raw_str, rem_str, file_name)
 
-                log.debug(c)
-                exit_code = os.system(c)
+                exit_code = os_utils.execute(c)
 
                 if exit_code != 0:
                     log.error("Print command failed with exit code %d!" % exit_code)
@@ -2413,8 +2437,6 @@ class Device(object):
 
 
     def printData(self, data, printer_name=None, direct=True, raw=True):
-        #log.log_data(data)
-        #log.debug("printData(direct=%s, raw=%s)" % (direct, raw))
         if direct:
             self.writePrint(data)
         else:
@@ -2437,7 +2459,7 @@ class Device(object):
         if self.dbus_avail:
             try:
                 device_uri, history = self.service.GetHistory(self.device_uri)
-            except dbus.exceptions.DBusException, e:
+            except dbus.exceptions.DBusException as e:
                 log.error("dbus call to GetHistory() failed.")
                 return []
 
@@ -2477,13 +2499,15 @@ class Device(object):
             opener = LocalOpener({})
             try:
                 f = opener.open(url2, data)
+                
             except Error:
                 log.error("Status read failed: %s" % url2)
                 stream.seek(0)
                 stream.truncate()
             else:
                 try:
-                    stream.write(f.read())
+                    stream.write(f.fp.read())
+                    #stream.write(f)
                 finally:
                     f.close()
 
@@ -2522,13 +2546,13 @@ class Device(object):
             self.closeLEDM()
 
     def FetchLEDMUrl(self, url, footer=""):
-        data_fp = cStringIO.StringIO()
+        data_fp = BytesIO()
         if footer:
             data = self.getUrl_LEDM(url, data_fp, footer)
         else:
             data = self.getUrl_LEDM(url, data_fp)
         if data:
-            data = data.split('\r\n\r\n', 1)[1]
+            data = data.split(to_bytes_utf8('\r\n\r\n'), 1)[1]
             if data:
                 data = status.ExtractXMLData(data)
         return data
@@ -2536,26 +2560,26 @@ class Device(object):
 #-------------------------For LEDM SOAP PROTOCOL(FAX) Devices----------------------------------------------------------------------#
 
     def FetchEWS_LEDMUrl(self, url, footer=""):
-        data_fp = cStringIO.StringIO()
+        data_fp = BytesIO()
         if footer:
             data = self.getEWSUrl_LEDM(url, data_fp, footer)
         else:
             data = self.getEWSUrl_LEDM(url, data_fp)
         if data:
-            data = data.split('\r\n\r\n', 1)[1]
+            data = data.split(to_bytes_utf8('\r\n\r\n'), 1)[1]
             if data:
                 data = status.ExtractXMLData(data)
         return data
 
     def readAttributeFromXml_EWS(self, uri, attribute):
-        stream = cStringIO.StringIO()
+        stream = BytesIO()
         data = self.FetchEWS_LEDMUrl(uri)
         if not data:
             log.error("Unable To read the XML data from device")
             return ""
         xmlDict = utils.XMLToDictParser().parseXML(data)
+
         try:
-            #return str(xmlDict[attribute])
             return xmlDict[attribute]
         except:
             return str("")
@@ -2563,17 +2587,18 @@ class Device(object):
 #---------------------------------------------------------------------------------------------------#
 
     def readAttributeFromXml(self,uri,attribute):
-        stream = cStringIO.StringIO()
+        stream = BytesIO()
         data = self.FetchLEDMUrl(uri)
         if not data:
             log.error("Unable To read the XML data from device")
             return ""
-        xmlDict = utils.XMLToDictParser().parseXML(data)
+        xmlDict = utils.XMLToDictParser().parseXML(data )
         try:
-            return str(xmlDict[attribute])
+            return xmlDict[attribute]
         except:
             return str("")
 
+
     def downloadFirmware(self, usb_bus_id=None, usb_device_id=None): # Note: IDs not currently used
         ok = False
         filename = os.path.join(prop.data_dir, "firmware", self.model.lower() + '.fw.gz')
@@ -2594,7 +2619,7 @@ class Device(object):
                     os.close(f)
                     ok = True
                     log.debug("OK")
-                except (OSError, IOError), e:
+                except (OSError, IOError) as e:
                     log.error("An error occured: %s" % e)
             else:
                 try:
@@ -2604,7 +2629,7 @@ class Device(object):
                     self.closePrint()
                     ok = True
                     log.debug("OK")
-                except Error, e:
+                except Error as e:
                     log.error("An error occured: %s" % e.msg)
         else:
             log.error("Firmware file '%s' not found." % filename)
@@ -2612,15 +2637,12 @@ class Device(object):
         return ok
 
 
-# ********************************** Support classes/functions
 
+    
 
-class xStringIO(StringIO.StringIO):
-    def makefile(self, x, y):
-        return self
 
 # URLs: hp:/usb/HP_LaserJet_3050?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
-class LocalOpener(urllib.URLopener):
+class LocalOpener(urllib_request.URLopener):
     def open_hp(self, url, dev):
         log.debug("open_hp(%s)" % url)
 
@@ -2635,23 +2657,22 @@ class LocalOpener(urllib.URLopener):
         dev.writeEWS("""GET %s HTTP/1.0\nContent-Length:0\nHost:localhost\nUser-Agent:hplip\n\n""" % loc)
 
         reply = xStringIO()
-
         while dev.readEWS(8192, reply, timeout=1):
             pass
 
         reply.seek(0)
         log.log_data(reply.getvalue())
-
-        response = httplib.HTTPResponse(reply)
+        
+        response = http_client.HTTPResponse(reply)
         response.begin()
 
-        if response.status != httplib.OK:
+        if response.status != http_client.OK:
             raise Error(ERROR_DEVICE_STATUS_NOT_AVAILABLE)
         else:
-            return response.fp
+            return response#.fp
 
 # URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
-class LocalOpenerEWS_LEDM(urllib.URLopener):
+class LocalOpenerEWS_LEDM(urllib_request.URLopener):
     def open_hp(self, url, dev, foot=""):
         log.debug("open_hp(%s)" % url)
 
@@ -2670,45 +2691,14 @@ class LocalOpenerEWS_LEDM(urllib.URLopener):
 
         reply = xStringIO()
 
-        #while dev.readEWS_LEDM(512, reply, timeout=3):
-            #pass
+        dev.readLEDMData(dev.readEWS_LEDM,reply)
 
-        END_OF_DATA="0\r\n\r\n"
-        bytes_requested = 1024        
-        bytes_remaining = 0
-        chunkedFlag = True
-
-        bytes_read = dev.readEWS_LEDM(bytes_requested, reply, timeout=3)
-
-        for line in reply.getvalue().splitlines():
-            if line.lower().find("content-length") != -1:   
-                 bytes_remaining = int(line.split(":")[1])
-                 chunkedFlag = False
-                 break
-
-        xml_data_start = reply.getvalue().find("<?xml")
-        if (xml_data_start != -1):
-            bytes_remaining = bytes_remaining - (len(reply.getvalue())  - xml_data_start)
-
-
-        while bytes_read > 0:
-            temp_buf = xStringIO()
-            bytes_read = dev.readEWS_LEDM(bytes_requested, temp_buf, timeout=3)
-            reply.write(temp_buf.getvalue())
-
-            if not chunkedFlag:     # Unchunked data
-                bytes_remaining = bytes_remaining - bytes_read
-                if bytes_remaining <= 0:   
-                    break
-            elif END_OF_DATA == temp_buf.getvalue():   # Chunked data end
-                    break        
-
-        reply.seek(0)  
+        reply.seek(0)
         return reply.getvalue()
 
 
 # URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
-class LocalOpener_LEDM(urllib.URLopener):
+class LocalOpener_LEDM(urllib_request.URLopener):
     def open_hp(self, url, dev, foot=""):
         log.debug("open_hp(%s)" % url)
 
@@ -2726,38 +2716,9 @@ class LocalOpener_LEDM(urllib.URLopener):
             dev.writeLEDM("""GET %s HTTP/1.1\r\nAccept: text/plain\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n""" % loc)
 
         reply = xStringIO()
- 
-        #while dev.readLEDM(512, reply, timeout=3):
-            #pass
 
-        END_OF_DATA="0\r\n\r\n"
-        bytes_requested = 1024        
-        bytes_remaining = 0
-        chunkedFlag = True
+       
+        dev.readLEDMData(dev.readLEDM,reply)
 
-        bytes_read = dev.readLEDM(bytes_requested, reply, timeout=3)
-        
-        for line in reply.getvalue().splitlines():
-            if line.lower().find("content-length") != -1:   
-                 bytes_remaining = int(line.split(":")[1])
-                 chunkedFlag = False
-                 break
-
-        xml_data_start = reply.getvalue().find("<?xml")
-        if (xml_data_start != -1):
-            bytes_remaining = bytes_remaining - (len(reply.getvalue())  - xml_data_start)
-
-        while bytes_read > 0:
-            temp_buf = xStringIO()
-            bytes_read = dev.readLEDM(bytes_requested, temp_buf, timeout=3)
-            reply.write(temp_buf.getvalue())
-
-            if not chunkedFlag:     # Unchunked data
-                bytes_remaining = bytes_remaining - bytes_read
-                if bytes_remaining <= 0:   
-                    break
-            elif END_OF_DATA == temp_buf.getvalue():   # Chunked data end
-                    break        
-
-        reply.seek(0)  
+        reply.seek(0)
         return reply.getvalue()
