@@ -1,3 +1,4 @@
+#!/usr/local/bin/python2.7
 # -*- coding: utf-8 -*-
 #
 # (c) Copyright @ 2013 Hewlett-Packard Development Company, L.P.
@@ -20,14 +21,15 @@
 #
 import os
 import getpass
-import cStringIO
 import time
 import string
 import platform
 
-from base import utils, tui, os_utils
-from base.g import *
-import pexpect
+from . import utils, tui
+from .g import *
+from .sixext import BytesIO, StringIO
+from .sixext.moves import input
+from . import pexpect
 
 PASSWORD_RETRY_COUNT = 3
 
@@ -61,10 +63,10 @@ AUTH_TYPES ={'mepis':'su',
 # This function promts for the username and password and returns (username,password)
 def showPasswordPrompt(prompt):
     import getpass
-    print ""
-    print ""
-    print log.bold(prompt)
-    username = raw_input("Username: ")
+    print ("")
+    print ("")
+    print (log.bold(prompt))
+    username = input("Username: ")
     password = getpass.getpass("Password: ")
 
     return (username, password)
@@ -73,11 +75,17 @@ def showPasswordPrompt(prompt):
 
 #TBD this function shoud be removed once distro class implemented
 def get_distro_name():
-    os_name = None;
-    if utils.which('lsb_release'):
-       name = os.popen('lsb_release -i | cut -f 2')
-       os_name = name.read().strip()
-       name.close()
+    os_name = None
+    try:
+        import platform
+        os_name = platform.dist()[0]
+    except ImportError:
+        os_name = None
+
+    if not os_name: 
+        name = os.popen('lsb_release -i | cut -f 2')
+        os_name = name.read().strip()
+        name.close()
 
     if not os_name:
        name = os.popen("cat /etc/issue 2>/dev/null | awk '{print $1}' | head -n 1")
@@ -87,9 +95,10 @@ def get_distro_name():
     if not os_name:
         os_name = platform.system()
 
-    if "redhatenterprise" in os_name.lower():
+    os_name = os_name.lower()
+    if "redhatenterprise" in os_name:
         os_name = 'rhel'
-    elif "suse" in os_name.lower():
+    elif "suse" in os_name:
         os_name = 'suse'
 
     return os_name
@@ -155,7 +164,6 @@ class Password(object):
                 pswd_msg = "Please enter the root/superuser password: "
             else:
                 pswd_msg = "Please enter the sudoer (%s)'s password: " % os.getenv('USER')
-
         return getpass.getpass(log.bold(pswd_msg))
 
 
@@ -178,11 +186,12 @@ class Password(object):
 
 
     def __password_check(self, cmd, timeout=1):
-        output = cStringIO.StringIO()
+        import io
+        output = io.StringIO()
         ok, ret = False, ''
 
         try:
-            child = pexpect.spawn(cmd, timeout=timeout)
+            child = pexpect.spawnu(cmd, timeout=timeout)
         except pexpect.ExceptionPexpect:
             return 1, ''
 
@@ -193,13 +202,12 @@ class Password(object):
                 while True:
                     update_spinner()
 
-                    i = child.expect_list(self.__expectList)
+                    i = child.expect(self.__expectList)
+                    
                     cb = child.before
                     if cb:
-                        # output
+
                         start = time.time()
-                        log.log_to_file(cb)
-                        #log.debug(cb)
                         output.write(cb)
 
                     if i == 0: # EOF
@@ -207,12 +215,13 @@ class Password(object):
                         break
 
                     elif i == 1: # TIMEOUT
+                        
                         continue
 
                     else: # password
                         child.sendline(self.__password)
 
-            except (Exception, pexpect.ExceptionPexpect):
+            except (Exception, pexpect.ExceptionPexpect) as e:          
                 log.exception()
 
         finally:
@@ -226,6 +235,7 @@ class Password(object):
         if ok:
             return child.exitstatus, ret
         else:
+            
             return 1, ''
 
 
@@ -321,7 +331,7 @@ class Password(object):
         self.__password =""
         self.__passwordValidated = False
         if self.__authType == 'sudo':
-            os_utils.execute("sudo -K")
+            utils.run("sudo -K")
 
 
     def getAuthType(self):
