@@ -42,7 +42,7 @@ try:
     devid_mode = '--id' in sys.argv # hack
     if devid_mode:
         log.set_level("none")
-        restrict = False
+
 
     mod = module.Module(__mod__, __title__, __version__, __doc__, None,
                         (INTERACTIVE_MODE, GUI_MODE), (UI_TOOLKIT_QT4,),
@@ -76,6 +76,9 @@ try:
 
     device_uri = mod.getDeviceUri(device_uri, printer_name, restrict_to_installed_devices=restrict)
 
+    if not device_uri:
+        sys.exit(1)
+
     if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
         try:
             d = device.Device(device_uri, printer_name)
@@ -92,7 +95,7 @@ try:
             try:
                 d.open()
                 d.queryDevice()
-            except Error, e:
+            except Error as e:
                 log.error("Error opening device (%s)." % e.msg)
                 #sys.exit(1)
 
@@ -107,12 +110,12 @@ try:
             if devid_mode:
                 try:
                     if d.dq['deviceid']:
-                        print(d.dq['deviceid'])
+                        print((d.dq['deviceid']))
                     sys.exit(0)
                 except KeyError:
                     log.error("Device ID not available.")
             else:
-                dq_keys = d.dq.keys()
+                dq_keys = list(d.dq.keys())
                 dq_keys.sort()
 
                 log.info(log.bold("Device Parameters (dynamic data):"))
@@ -126,7 +129,7 @@ try:
                 log.info(log.bold(formatter.compose(("Parameter", "Value(s)"))))
                 log.info(formatter.compose(('-'*28, '-'*58)))
 
-                mq_keys = d.mq.keys()
+                mq_keys = list(d.mq.keys())
                 mq_keys.sort()
 
                 for key in mq_keys:
@@ -168,7 +171,6 @@ try:
 
         if 1:
             app = QApplication(sys.argv)
-
             dlg = InfoDialog(None, device_uri)
             dlg.show()
             try:
