@@ -71,7 +71,7 @@ class Console(cmd.Cmd):
     # Command definitions
     def do_hist(self, args):
         """Print a list of commands that have been entered"""
-        print self._hist
+        print(self._hist)
 
     def do_exit(self, args):
         """Exits from the console"""
@@ -109,7 +109,7 @@ class Console(cmd.Cmd):
            Despite the claims in the Cmd documentaion, Cmd.postloop() is not a stub.
         """
         cmd.Cmd.postloop(self)   # Clean up command completion
-        print "Exiting..."
+        print("Exiting...")
 
     def precmd(self, line):
         """ This method is called after the line has been input but before
@@ -130,7 +130,7 @@ class Console(cmd.Cmd):
         pass
 
     def default(self, line):
-        print log.bold("ERROR: Unrecognized command. Use 'help' to list commands.")
+        print(log.bold("ERROR: Unrecognized command. Use 'help' to list commands."))
 
     def do_ldir(self, args):
         """ List local directory contents."""
@@ -158,22 +158,22 @@ class Console(cmd.Cmd):
                 )
             )
 
-        print
-        print log.bold(formatter.compose(("Name", "Size", "Type")))
+        print()
+        print(log.bold(formatter.compose(("Name", "Size", "Type"))))
 
         num_files = 0
         for d in self.pc.current_directories():
             if d[0] in ('.', '..'):
-                print formatter.compose((d[0], "", "directory"))
+                print(formatter.compose((d[0], "", "directory")))
             else:
-                print formatter.compose((d[0] + "/", "", "directory"))
+                print(formatter.compose((d[0] + "/", "", "directory")))
 
         for f in self.pc.current_files():
-            print formatter.compose((f[0], utils.format_bytes(f[2]), self.pc.classify_file(f[0])))
+            print(formatter.compose((f[0], utils.format_bytes(f[2]), self.pc.classify_file(f[0]))))
             num_files += 1
             total_size += f[2]
 
-        print log.bold("% d files, %s" % (num_files, utils.format_bytes(total_size, True)))
+        print(log.bold("% d files, %s" % (num_files, utils.format_bytes(total_size, True))))
 
 
     def do_df(self, args):
@@ -188,7 +188,7 @@ class Console(cmd.Cmd):
         else:
             fs = utils.commafy(freespace)
 
-        print "Freespace = %s Bytes" % fs
+        print("Freespace = %s Bytes" % fs)
 
 
     def do_cp(self, args, remove_after_copy=False):
@@ -204,11 +204,11 @@ class Console(cmd.Cmd):
         matched_files = self.pc.match_files(args)
 
         if len(matched_files) == 0:
-            print "ERROR: File(s) not found."
+            print("ERROR: File(s) not found.")
         else:
             total, delta = self.pc.cp_multiple(matched_files, remove_after_copy, self.cp_status_callback, self.rm_status_callback)
 
-            print log.bold("\n%s transfered in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/(delta)))
+            print(log.bold("\n%s transfered in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/(delta))))
 
     def do_unload(self, args):
         """Unload all image files from photocard to current local directory.
@@ -228,7 +228,7 @@ class Console(cmd.Cmd):
 
 
         unload_list = self.pc.get_unload_list()
-        print
+        print()
 
         if len(unload_list) > 0:
             if '-p' in args:
@@ -245,34 +245,34 @@ class Console(cmd.Cmd):
                         )
                     )
 
-                print
-                print log.bold(formatter.compose(("Name", "Size", "Type")))
+                print()
+                print(log.bold(formatter.compose(("Name", "Size", "Type"))))
 
                 total = 0
                 for u in unload_list:
-                     print formatter.compose(('%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % (u[2], u[3])))
+                     print(formatter.compose(('%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % (u[2], u[3]))))
                      total += u[1]
 
 
-                print log.bold("Found %d files to unload, %s" % (len(unload_list), utils.format_bytes(total, True)))
+                print(log.bold("Found %d files to unload, %s" % (len(unload_list), utils.format_bytes(total, True))))
             else:
-                print log.bold("Unloading %d files..." % len(unload_list))
+                print(log.bold("Unloading %d files..." % len(unload_list)))
                 total, delta, was_cancelled = self.pc.unload(unload_list, self.cp_status_callback, self.rm_status_callback, dont_remove)
-                print log.bold("\n%s unloaded in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/delta))
+                print(log.bold("\n%s unloaded in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/delta)))
 
         else:
-            print "No image, audio, or video files found."
+            print("No image, audio, or video files found.")
 
 
     def cp_status_callback(self, src, trg, size):
         if size == 1:
-            print
-            print log.bold("Copying %s..." % src)
+            print()
+            print(log.bold("Copying %s..." % src))
         else:
-            print "\nCopied %s to %s (%s)..." % (src, trg, utils.format_bytes(size))
+            print("\nCopied %s to %s (%s)..." % (src, trg, utils.format_bytes(size)))
 
     def rm_status_callback(self, src):
-        print "Removing %s..." % src
+        print("Removing %s..." % src)
 
 
 
@@ -287,7 +287,7 @@ class Console(cmd.Cmd):
         matched_files = self.pc.match_files(args)
 
         if len(matched_files) == 0:
-            print "ERROR: File(s) not found."
+            print("ERROR: File(s) not found.")
         else:
             for f in matched_files:
                 self.pc.rm(f, False)
@@ -303,21 +303,21 @@ class Console(cmd.Cmd):
 
     def do_lpwd(self, args):
         """Print name of local current/working directory."""
-        print os.getcwd()
+        print(os.getcwd())
 
     def do_lcd(self, args):
         """Change current local working directory."""
         try:
             os.chdir(args.strip())
         except OSError:
-            print log.bold("ERROR: Directory not found.")
-        print os.getcwd()
+            print(log.bold("ERROR: Directory not found."))
+        print(os.getcwd())
 
     def do_pwd(self, args):
         """Print name of photo card current/working directory
         Usage:
         \t>pwd"""
-        print self.pc.pwd()
+        print(self.pc.pwd())
 
     def do_cd(self, args):
         """Change current working directory on photo card.
@@ -342,10 +342,10 @@ class Console(cmd.Cmd):
             matched_dirs = self.pc.match_dirs(args)
 
             if len(matched_dirs) == 0:
-                print "Directory not found"
+                print("Directory not found")
 
             elif len(matched_dirs) > 1:
-                print "Pattern matches more than one directory"
+                print("Pattern matches more than one directory")
 
             else:
                 self.pc.cd(matched_dirs[0])
@@ -379,16 +379,16 @@ class Console(cmd.Cmd):
             if self.pc.cache_state():
                 cache_info = self.pc.cache_info()
 
-                t = cache_info.keys()
+                t = list(cache_info.keys())
                 t.sort()
-                print
+                print()
                 for s in t:
-                    print "sector %d (%d hits)" % (s, cache_info[s])
+                    print("sector %d (%d hits)" % (s, cache_info[s]))
 
-                print log.bold("Total cache usage: %s (%s maximum)" % (utils.format_bytes(len(t)*512), utils.format_bytes(photocard.MAX_CACHE * 512)))
-                print log.bold("Total cache sectors: %s of %s" % (utils.commafy(len(t)), utils.commafy(photocard.MAX_CACHE)))
+                print(log.bold("Total cache usage: %s (%s maximum)" % (utils.format_bytes(len(t)*512), utils.format_bytes(photocard.MAX_CACHE * 512))))
+                print(log.bold("Total cache sectors: %s of %s" % (utils.commafy(len(t)), utils.commafy(photocard.MAX_CACHE))))
             else:
-                print "Cache is off."
+                print("Cache is off.")
 
     def do_sector(self, args):
         """Display sector data.
@@ -400,25 +400,25 @@ class Console(cmd.Cmd):
         try:
             sector = int(args)
         except ValueError:
-            print "Sector must be specified as a number"
+            print("Sector must be specified as a number")
             return
 
         if self.pc.cache_check(sector) > 0:
-            print "Cached sector"
+            print("Cached sector")
 
-        print repr(self.pc.sector(sector))
+        print(repr(self.pc.sector(sector)))
 
 
     def do_tree(self, args):
         """Display photo card directory tree."""
         tree = self.pc.tree()
-        print
+        print()
         self.print_tree(tree)
 
     def print_tree(self, tree, level=0):
         for d in tree:
             if type(tree[d]) == type({}):
-                print ''.join([' '*level*4, d, '/'])
+                print(''.join([' '*level*4, d, '/']))
                 self.print_tree(tree[d], level+1)
 
 
@@ -429,21 +429,21 @@ class Console(cmd.Cmd):
 
     def do_card(self, args):
         """Print info about photocard."""
-        print
-        print "Device URI = %s" % self.pc.device.device_uri
-        print "Model = %s" % self.pc.device.model_ui
-        print "Working dir = %s" % self.pc.pwd()
+        print()
+        print("Device URI = %s" % self.pc.device.device_uri)
+        print("Model = %s" % self.pc.device.model_ui)
+        print("Working dir = %s" % self.pc.pwd())
         disk_info = self.pc.info()
-        print "OEM ID = %s" % disk_info[0]
-        print "Bytes/sector = %d" % disk_info[1]
-        print "Sectors/cluster = %d" % disk_info[2]
-        print "Reserved sectors = %d" % disk_info[3]
-        print "Root entries = %d" % disk_info[4]
-        print "Sectors/FAT = %d" % disk_info[5]
-        print "Volume label = %s" % disk_info[6]
-        print "System ID = %s" % disk_info[7]
-        print "Write protected = %d" % disk_info[8]
-        print "Cached sectors = %s" % utils.commafy(len(self.pc.cache_info()))
+        print("OEM ID = %s" % disk_info[0])
+        print("Bytes/sector = %d" % disk_info[1])
+        print("Sectors/cluster = %d" % disk_info[2])
+        print("Reserved sectors = %d" % disk_info[3])
+        print("Root entries = %d" % disk_info[4])
+        print("Sectors/FAT = %d" % disk_info[5])
+        print("Volume label = %s" % disk_info[6])
+        print("System ID = %s" % disk_info[7])
+        print("Write protected = %d" % disk_info[8])
+        print("Cached sectors = %s" % utils.commafy(len(self.pc.cache_info())))
 
 
     def do_display(self, args):
@@ -464,13 +464,13 @@ class Console(cmd.Cmd):
                 os.remove(temp_name)
 
             else:
-                print "File is not an image."
+                print("File is not an image.")
 
         elif len(matched_files) == 0:
-            print "File not found."
+            print("File not found.")
 
         else:
-            print "Only one file at a time may be specified for display."
+            print("Only one file at a time may be specified for display.")
 
     def do_show(self, args):
         """Synonym for the display command."""
@@ -507,15 +507,15 @@ class Console(cmd.Cmd):
                     os.remove(temp_file_name)
 
                 else:
-                    print "No thumbnail found."
+                    print("No thumbnail found.")
 
             else:
-                print "Incorrect file type for thumbnail."
+                print("Incorrect file type for thumbnail.")
 
         elif len(matched_files) == 0:
-            print "File not found."
+            print("File not found.")
         else:
-            print "Only one file at a time may be specified for thumbnail display."
+            print("Only one file at a time may be specified for thumbnail display.")
 
     def do_thumb(self, args):
         """Synonym for the thumbnail command."""
@@ -542,24 +542,24 @@ class Console(cmd.Cmd):
                         )
                     )
 
-                print
-                print log.bold(formatter.compose(("Tag", "Value")))
+                print()
+                print(log.bold(formatter.compose(("Tag", "Value"))))
 
-                ee = exif_info.keys()
+                ee = list(exif_info.keys())
                 ee.sort()
                 for e in ee:
                     if e not in ('JPEGThumbnail', 'TIFFThumbnail', 'Filename'):
                         #if e != 'EXIF MakerNote':
-                        print formatter.compose((e, '%s' % exif_info[e]))
+                        print(formatter.compose((e, '%s' % exif_info[e])))
                         #else:
                         #    print formatter.compose( ( e, ''.join( [ chr(x) for x in exif_info[e].values if chr(x) in string.printable ] ) ) )
             else:
-                print "Incorrect file type for thumbnail."
+                print("Incorrect file type for thumbnail.")
 
         elif len(matched_files) == 0:
-            print "File not found."
+            print("File not found.")
         else:
-            print "Only one file at a time may be specified for thumbnail display."
+            print("Only one file at a time may be specified for thumbnail display.")
 
     def do_info(self, args):
         """Synonym for the exif command."""
@@ -571,10 +571,10 @@ class Console(cmd.Cmd):
 
 def status_callback(src, trg, size):
     if size == 1:
-        print
-        print log.bold("Copying %s..." % src)
+        print()
+        print(log.bold("Copying %s..." % src))
     else:
-        print "\nCopied %s to %s (%s)..." % (src, trg, utils.format_bytes(size))
+        print("\nCopied %s to %s (%s)..." % (src, trg, utils.format_bytes(size)))
 
 
 
@@ -609,9 +609,12 @@ if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
         device_uri = mod.getDeviceUri(device_uri, printer_name,
             filter={'pcard-type' : (operator.eq, 1)})
 
+        if not device_uri:
+            sys.exit(1)
+
         try:
             pc = photocard.PhotoCard( None, device_uri, printer_name )
-        except Error, e:
+        except Error as e:
             log.error("Unable to start photocard session: %s" % e.msg)
             sys.exit(1)
 
@@ -634,7 +637,7 @@ if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
         try:
             os.chdir(output_dir)
         except OSError:
-            print log.bold("ERROR: Output directory %s not found." % output_dir)
+            print(log.bold("ERROR: Output directory %s not found." % output_dir))
             sys.exit(1)
 
 
@@ -645,7 +648,7 @@ if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
                     console . cmdloop()
                 except KeyboardInterrupt:
                     log.error("Aborted.")
-                except Exception, e:
+                except Exception as e:
                     log.error("An error occured: %s" % e)
             finally:
                 pc.umount()
@@ -655,10 +658,10 @@ if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
 
 
         else: # NON_INTERACTIVE_MODE
-            print "Output directory is %s" % os.getcwd()
+            print("Output directory is %s" % os.getcwd())
             try:
                 unload_list = pc.get_unload_list()
-                print
+                print()
 
                 if len(unload_list) > 0:
 
@@ -674,19 +677,19 @@ if mode in (INTERACTIVE_MODE, NON_INTERACTIVE_MODE):
                             )
                         )
 
-                    print
-                    print log.bold(formatter.compose(("Name", "Size", "Type")))
+                    print()
+                    print(log.bold(formatter.compose(("Name", "Size", "Type"))))
 
                     total = 0
                     for u in unload_list:
-                         print formatter.compose(('%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % (u[2], u[3])))
+                         print(formatter.compose(('%s' % u[0], utils.format_bytes(u[1]), '%s/%s' % (u[2], u[3]))))
                          total += u[1]
 
 
-                    print log.bold("Found %d files to unload, %s\n" % (len(unload_list), utils.format_bytes(total, True)))
-                    print log.bold("Unloading files...\n")
+                    print(log.bold("Found %d files to unload, %s\n" % (len(unload_list), utils.format_bytes(total, True))))
+                    print(log.bold("Unloading files...\n"))
                     total, delta, was_cancelled = pc.unload(unload_list, status_callback, None, True)
-                    print log.bold("\n%s unloaded in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/delta))
+                    print(log.bold("\n%s unloaded in %d sec (%d KB/sec)" % (utils.format_bytes(total), delta, (total/1024)/delta)))
 
 
             finally:
